package krusty;

import spark.Request;
import spark.Response;

// Likely dependencies for db operations
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.io.BufferedReader;
// Likely dependencies for general operations
import java.io.IOException;
import java.io.FileReader;
import java.sql.ResultSet;
import java.util.StringJoiner;


public class Database {
	// Here, we use an in-memory database. This string could be changed to
	// "jdbc:sqlite:<filename>.sqlite3" to use a file-based database instead.
	// Nore that ":memory:" is an **SQLite specific** magic string that tells the
	// underlying SQLite engine to store the database in memory.
	private static final String jdbcString = "jdbc:sqlite::memory:";
	private Connection conn = null;

	public String getCustomers(Request req, Response res) {
		String result = selectQuery("Customers", "customers", "name", "address");
		return result;
	}

	public String getRawMaterials(Request req, Response res) {
		return "{}";
	}

	public String getCookies(Request req, Response res) {
		return "{\"cookies\":[]}";
	}

	public String getRecipes(Request req, Response res) {
		return "{}";
	}

	public String getPallets(Request req, Response res) {
		return "{\"pallets\":[]}";
	}

	public String reset(Request req, Response res) {
		return "{}";
	}

	public String createPallet(Request req, Response res) {
		return "{}";
	}

	/** Connects to the database using the configured jdbcString. */
	public void connect() {
		try {
			conn = DriverManager.getConnection(jdbcString);
		} catch (Exception e) {
			System.out.println(e.getMessage());
		} finally {
			System.out.println("Connection to SQLite has been established.");
		}
	}

	/**
	 * Selects columns from a table and returns the result as a JSON string.
	 * Does _absolutely no_ query sanitization, so be careful with user input.
	 */
	private String selectQuery(String table, String jsonName, String... columns) {
		String jsonResult = "{}"; // Valid json to return if fail

		try {
			Statement stmt = this.conn.createStatement();
			StringBuilder query = new StringBuilder("SELECT ");

			StringJoiner args = new StringJoiner(", ");
			for (String column : columns) {
				args.add(column);
			}

			query.append(args.toString());
			query.append("\nFROM " + table + ";");

			/* Sanitization is for cowards */

			ResultSet result = stmt.executeQuery(query.toString());
			jsonResult = Jsonizer.toJson(result, jsonName);
		} catch (SQLException e) {
			System.out.printf("Error executing query: \n%s", e);
		}

		return jsonResult;
	}

	// The script location is relative to the gradle
	// build script ("build.gradle.kts", in this case).
	/** Reads an sql script into the database */
	public void migrateScript(String filename) throws IOException, SQLException {
		try (BufferedReader reader = new BufferedReader(new FileReader(filename))) {
			StringBuilder scriptBuilder = new StringBuilder();
			String line;

			// Read the script file line by line
			while ((line = reader.readLine()) != null) {
				scriptBuilder.append(line).append("\n");
			}
			String script = scriptBuilder.toString().trim();

			// Execute the script
			try (Statement statement = conn.createStatement()) {
				statement.execute(script);
			}
			System.out.println(String.format("Executed script %s", filename));
		} catch (IOException e) {
			System.err.println("Error reading script file: " + e.getMessage());
			throw e;
		} catch (SQLException e) {
			System.err.println("Error executing script: " + e.getMessage());
			throw e;
		}
	}
}
